import os
import base64

from django.shortcuts import render, redirect
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.core.files.base import ContentFile
from django.core.files.storage import default_storage
from django.core.files.storage import FileSystemStorage
from django.http import HttpResponse, HttpResponseRedirect
from django.template.defaultfilters import slugify
from django.utils.crypto import get_random_string
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from rest_framework.parsers import FileUploadParser
from rest_framework.response import Response
from rest_framework.views import APIView
from djangoldp.utils import is_authenticated_user

from djangoldp_uploader.forms import DocumentForm
from djangoldp_uploader.models import Document, File


def home(request):
    documents = Document.objects.all()
    return render(request, 'home.html', { 'documents': documents })


def demo_sib(request):
    return render(request, 'demo_sib.html')


def upload(request):
    if True: #is_authenticated_user(request.user): to delete when issue https://git.startinblox.com/framework/sib-core/issues/741 closed
        try:
            file_obj = request.FILES['file']
            fs = FileSystemStorage()
            file_name, file_ext = os.path.splitext(file_obj.name)
            file_name = slugify(file_name) + file_ext
            filename = fs.save(file_name, file_obj)
            uploaded_file_url = fs.url(filename)

            return Response(status=204, headers={'Location': "{}{}".format(settings.SITE_URL, uploaded_file_url)})

        except:
            return HttpResponse(status=400)

    else:
        return HttpResponse(status=403)


def compose_filename(hashcode, filename, fileext):
    file_name = '{}_slash_{}{}'.format(hashcode, filename, fileext)
    return file_name


def upload_put(request, hashcode, filename, fileext):
    if is_authenticated_user(request.user):
        try:
            file_name = compose_filename(hashcode, filename, fileext)
            file_obj = request.FILES['file']
            request_path = request.path
            content_type = request.content_type
            uploaded_file_url = settings.MEDIA_URL + file_name

            with open(uploaded_file_url[1:], 'wb+') as f:
                for chunk in file_obj.chunks():
                    f.write(chunk)

            file_model = File(
                stored_url = uploaded_file_url,
                original_url = request_path,
                content_type = content_type
            ).save()

            return HttpResponse(status=201)

        except:
            return HttpResponse(status=400)

    else:
        return HttpResponse(status=403)


class FileUploadPostView(APIView):
    def dispatch(self, request, *args, **kwargs):
        request.csrf_processing_done = True
        response = super().dispatch(request, *args, **kwargs)
        response["Access-Control-Allow-Origin"] = request.META.get('HTTP_ORIGIN')
        response["Access-Control-Allow-Methods"] = "POST,OPTIONS"
        response["Access-Control-Allow-Headers"] = "authorization, Content-Type, if-match, accept, sentry-trace, dpop"
        response["Access-Control-Expose-Headers"] = "Location, User"
        response["Access-Control-Allow-Credentials"] = 'true'

        return response

    def post(self, request):
        if request.FILES['file']:
            response = upload(request)
            
            return response

        else:
            return HttpResponse('No file provided', status=400)

class FileUploadView(APIView):
    parser_classes = (FileUploadParser,)

    def dispatch(self, request, *args, **kwargs):
        request.csrf_processing_done = True
        '''overriden dispatch method to append some custom headers'''
        response = super().dispatch(request, *args, **kwargs)
        response["Access-Control-Allow-Origin"] = request.META.get('HTTP_ORIGIN')
        response["Access-Control-Allow-Methods"] = "GET,POST,PUT,OPTIONS"
        response["Access-Control-Allow-Headers"] = "authorization, Content-Type, if-match, accept, sentry-trace, dpop"
        response["Access-Control-Expose-Headers"] = "Location, User"
        response["Access-Control-Allow-Credentials"] = 'true'

        return response


    def options(self, request, hashcode, filename, fileext):
        return HttpResponse(status=204)


    def post(self, request):
        if request.FILES['file']:
            response = upload(request)

            return response

        else:
            return HttpResponse('No file provided', status=400)


    def put(self, request, hashcode, filename, fileext):
        filename, fileext = os.path.splitext(request.path)
        filename = slugify(filename.split('/')[-1])

        if request.FILES['file']:
            if File.objects.filter(original_url = request.path).exists():
                return HttpResponse('File already existing at URL', status=409)
            
            else:
                response = upload_put(request, hashcode, filename, fileext)

                return response

        else:
            return self.get(request, hashcode, filename, fileext)


    def get(self, request, hashcode, filename, fileext):
        if File.objects.filter(original_url = request.path).exists():
            file_model = File.objects.get(original_url = request.path)
            return redirect(file_model.stored_url)
                
        else:
            return HttpResponse('No file found at URL', status=404)


def upload_view(request):
    if request.method == 'POST' and request.FILES['file']:
        uploaded_file_url = upload(request)
        return render(request, 'simple_upload.html', {
            'uploaded_file_url': uploaded_file_url
        })
    return render(request, 'simple_upload.html')


def model_form_upload(request):
    if request.method == 'POST':
        form = DocumentForm(request.POST, request.FILES)
        if form.is_valid():
            form.save()
            return redirect('home')
    else:
        form = DocumentForm()
    return render(request, 'model_form_upload.html', {
        'form': form
    })



# Generated by Django 2.2.17 on 2021-02-10 08:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_uploader', '0012_file_document'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='file',
            name='document',
        ),
        migrations.RemoveField(
            model_name='file',
            name='is_bytes',
        ),
        migrations.RemoveField(
            model_name='file',
            name='object_content',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2019-07-03 09:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Document',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(blank=True, max_length=255)),
                ('document', models.FileField(upload_to='documents/')),
                ('uploaded_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'control'),
            },
        ),
    ]



"""djangoldp uploader URL Configuration"""

from django.conf import settings
from django.conf.urls import url
from django.conf.urls.static import static
from django.views.decorators.csrf import csrf_exempt

from djangoldp_uploader import views
from djangoldp_uploader.views import FileUploadView, FileUploadPostView

urlpatterns = [
    url(r'^upload/$', csrf_exempt(FileUploadPostView.as_view()), name='upload'),
    url(r'^upload/(?P<hashcode>[\w|\W-]+)/(?P<filename>[\w|\W-]+).(?P<fileext>[\w]+)$', csrf_exempt(FileUploadView.as_view()), name='upload_xmpp')
]

urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

if settings.DEBUG:
    urlpatterns += [
        url(r'^demo/$', views.home, name='home'),
        url(r'^demo/sib/$', views.demo_sib, name='upload_sib'),
        url(r'^demo/simple/$', views.upload_view, name='simple_upload'),
        url(r'^demo/form/$', views.model_form_upload, name='model_form_upload'),
    ]



# Generated by Django 2.2.17 on 2021-02-08 08:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_uploader', '0009_auto_20210202_1510'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='document',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control']},
        ),
        migrations.AlterModelOptions(
            name='file',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control']},
        ),
        migrations.AddField(
            model_name='file',
            name='is_bytes',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='file',
            name='object_content',
            field=models.BinaryField(blank=True),
        ),
        migrations.AlterField(
            model_name='file',
            name='stored_url',
            field=models.CharField(blank=True, max_length=255),
        ),
    ]



# Generated by Django 2.2.17 on 2021-02-16 15:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_uploader', '0013_auto_20210210_0913'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='document',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control']},
        ),
        migrations.AlterModelOptions(
            name='file',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control']},
        ),
    ]



# Generated by Django 2.2.17 on 2021-02-09 08:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_uploader', '0011_auto_20210208_1801'),
    ]

    operations = [
        migrations.AddField(
            model_name='file',
            name='document',
            field=models.FileField(default=None, upload_to='media/'),
        ),
    ]



import sys
import yaml

import django
from django.conf import settings as django_settings
from djangoldp.conf.ldpsettings import LDPSettings

# create a test configuration
config = {
    # add the packages to the reference list
    'ldppackages': ['djangoldp_uploader'],

    'MEDIA_URL': '/media/',
    'MEDIA_ROOT': 'media',

    # required values for server
    'server': {
        'MEDIA_URL': '/media/',
        'MEDIA_ROOT': 'media',
    }
}

ldpsettings = LDPSettings(config)
django_settings.configure(ldpsettings)

django.setup()
from django.test.runner import DiscoverRunner

test_runner = DiscoverRunner(verbosity=1)

failures = test_runner.run_tests([
    'djangoldp_uploader.tests.tests_upload',
])
if failures:
    sys.exit(failures)



# Generated by Django 2.2.17 on 2021-01-19 13:35

from django.db import migrations, models
import djangoldp.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_uploader', '0005_auto_20200619_0811'),
    ]

    operations = [
        migrations.CreateModel(
            name='File',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('stored_url', models.CharField(max_length=255)),
                ('original_url', models.CharField(max_length=255)),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'control'),
                'depth': 0,
            },
        ),
    ]



from django import forms

from djangoldp_uploader.models import Document


class DocumentForm(forms.ModelForm):
    class Meta:
        model = Document
        fields = ('description', 'document', )



# Generated by Django 2.2.17 on 2021-02-02 09:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_uploader', '0007_auto_20210128_0724'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='document',
            options={'default_permissions': ('add', 'change', 'delete', 'view', 'control')},
        ),
        migrations.AlterModelOptions(
            name='file',
            options={'default_permissions': ('add', 'change', 'delete', 'view', 'control')},
        ),
    ]



from django.conf import settings
from djangoldp.models import Model
from django.db import models


class Document(Model):
    description = models.CharField(max_length=255, blank=True)
    document = models.FileField(upload_to='documents/')
    uploaded_at = models.DateTimeField(auto_now_add=True)


class File(Model):
    original_url = models.CharField(max_length=255)
    content_type = models.CharField(max_length=255, default='None')
    stored_url = models.CharField(max_length=255, blank=True)

    def __str__(self):
        return self.original_url


import json
import urllib
from django.conf import settings
from django.core.files.uploadedfile import SimpleUploadedFile
from django.utils.crypto import get_random_string
from datetime import datetime, timedelta
from rest_framework.test import APITestCase, APIClient

from djangoldp_uploader.models import File


class TestUPLOAD(APITestCase):

    def setUp(self):
        self.client = APIClient()

    def test_get_uploaded(self):
        file_name = 'file.txt'
        file_url = '/upload/{}/{}'.format(get_random_string(32), file_name)
        sample_file = SimpleUploadedFile(file_name, b"file_content")

        response = self.client.put(file_url, {'file':sample_file})
        self.assertEqual(response.status_code, 201)

        response = self.client.get(file_url, follow=True)
        # self.assertEqual(response.status_code, 200)
        


# Generated by Django 2.2.17 on 2021-02-02 14:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_uploader', '0008_auto_20210202_1058'),
    ]

    operations = [
        migrations.AddField(
            model_name='file',
            name='content_type',
            field=models.CharField(default='None', max_length=255),
        ),
        migrations.AddField(
            model_name='file',
            name='object_content',
            field=models.BinaryField(default=b'None'),
        ),
    ]
